@@ -39,6 +39,7 @@ class Puppet::Application::Agent < Puppet::Application
       :graph => true,
       :fingerprint => false,
       :sourceaddress => nil,
+      :start_time => Time.now,
     }.each do |opt,val|
       options[opt] = val
     end
@@ -90,7 +91,7 @@ puppet-agent(8) -- #{summary}
 
 SYNOPSIS
 --------
-Retrieves the client configuration from the puppet master and applies it to
+Retrieves the client configuration from the Puppet master and applies it to
 the local host.
 
 This service may be run as a daemon, run periodically using cron (or something
@@ -102,7 +103,7 @@ USAGE
 puppet agent [--certname <NAME>] [-D|--daemonize|--no-daemonize]
   [-d|--debug] [--detailed-exitcodes] [--digest <DIGEST>] [--disable [MESSAGE]] [--enable]
   [--fingerprint] [-h|--help] [-l|--logdest syslog|eventlog|<ABS FILEPATH>|console]
-  [--masterport <PORT>] [--noop] [-o|--onetime] [--sourceaddress <IP_ADDRESS>] [-t|--test]
+  [--serverport <PORT>] [--noop] [-o|--onetime] [--sourceaddress <IP_ADDRESS>] [-t|--test]
   [-v|--verbose] [-V|--version] [-w|--waitforcert <SECONDS>]
 
 
@@ -124,31 +125,38 @@ configuration and apply it.
 USAGE NOTES
 -----------
 'puppet agent' does its best to find a compromise between interactive
-use and daemon use. Run with no arguments and no configuration, it will
-go into the background, attempt to get a signed certificate, and retrieve
-and apply its configuration every 30 minutes.
+use and daemon use. If you run it with no arguments and no configuration, it
+goes into the background, attempts to get a signed certificate, and retrieves
+and applies its configuration every 30 minutes.
 
-Some flags are meant specifically for interactive use -- in particular,
+Some flags are meant specifically for interactive use --- in particular,
 'test', 'tags' and 'fingerprint' are useful.
 
-'--test' does a single run in the foreground with verbose logging, then exits.
-It will also exit if it can't get a valid catalog. The exit code after running
-with '--test' is 0 if the catalog was successfully applied, and 1 if the run
-either failed or wasn't attempted (due to another run already in progress).
+'--test' runs once in the foreground with verbose logging, then exits.
+It also exits if it can't get a valid catalog. `--test` includes the 
+'--detailed-exitcodes' option by default and exits with one of the following 
+exit codes:
+
+* 0: The run succeeded with no changes or failures; the system was already in 
+     the desired state.
+* 1: The run failed, or wasn't attempted due to another run already in progress.
+* 2: The run succeeded, and some resources were changed.
+* 4: The run succeeded, and some resources failed.
+* 6: The run succeeded, and included both changes and failures.
 
 '--tags' allows you to specify what portions of a configuration you want
 to apply. Puppet elements are tagged with all of the class or definition
 names that contain them, and you can use the 'tags' flag to specify one
 of these names, causing only configuration elements contained within
 that class or definition to be applied. This is very useful when you are
-testing new configurations -- for instance, if you are just starting to
+testing new configurations --- for instance, if you are just starting to
 manage 'ntpd', you would put all of the new elements into an 'ntpd'
 class, and call puppet with '--tags ntpd', which would only apply that
 small portion of the configuration during your testing, rather than
 applying the whole thing.
 
-'--fingerprint' is a one-time flag. In this mode 'puppet agent' will run
-once and display on the console (and in the log) the current certificate
+'--fingerprint' is a one-time flag. In this mode 'puppet agent' runs
+once and displays on the console (and in the log) the current certificate
 (or certificate request) fingerprint. Providing the '--digest' option
 allows to use a different digest algorithm to generate the fingerprint.
 The main use is to verify that before signing a certificate request on
@@ -156,14 +164,19 @@ the master, the certificate request the master received is the same as
 the one the client sent (to prevent against man-in-the-middle attacks
 when signing certificates).
 
+'--skip_tags' is a flag used to filter resources. If this is set, then
+only resources not tagged with the specified tags will be applied.
+Values must be comma-separated.
+
 
 OPTIONS
 -------
 
 Note that any Puppet setting that's valid in the configuration file is also a
 valid long argument. For example, 'server' is a valid setting, so you can
-specify '--server <servername>' as an argument. Boolean settings translate into
-'--setting' and '--no-setting' pairs.
+specify '--server <servername>' as an argument. Boolean settings accept a '--no-' 
+prefix to turn off a behavior, translating into '--setting' and '--no-setting' 
+pairs, such as `--daemonize` and `--no-daemonize`.
 
 See the configuration file documentation at
 https://puppet.com/docs/puppet/latest/configuration.html for the
@@ -192,8 +205,8 @@ generated by running puppet agent with '--genconfig'.
   Enable full debugging.
 
 * --detailed-exitcodes:
-  Provide extra information about the run via exit codes; only works if '--test'
-  or '--onetime' is also specified. If enabled, 'puppet agent' will use the
+  Provide extra information about the run via exit codes; works only if '--test'
+  or '--onetime' is also specified. If enabled, 'puppet agent' uses the
   following exit codes:
 
   0: The run succeeded with no changes or failures; the system was already in
@@ -230,10 +243,15 @@ generated by running puppet agent with '--genconfig'.
 * --enable:
   Enable working on the local system. This removes any lock file,
   causing 'puppet agent' to start managing the local system again
-  (although it will continue to use its normal scheduling, so it might
-  not start for another half hour).
+  However, it continues to use its normal scheduling, so it might
+  not start for another half hour.
 
   'puppet agent' exits after executing this.
+  
+*  --evaltrace:
+  Logs each resource as it is being evaluated. This allows you to interactively
+  see exactly what is being done. (This is a Puppet setting, and can go in
+  puppet.conf. Note the special 'no-' prefix for boolean settings on the command line.)
 
 * --fingerprint:
   Display the current certificate or certificate signing request
@@ -254,19 +272,25 @@ generated by running puppet agent with '--genconfig'.
   service), 'eventlog' (the Windows Event Log), 'console', or the path to a log
   file. If debugging or verbosity is enabled, this defaults to 'console'.
   Otherwise, it defaults to 'syslog' on POSIX systems and 'eventlog' on Windows.
+  Multiple destinations can be set using a comma separated list 
+  (eg: `/path/file1,console,/path/file2`)"
 
   A path ending with '.json' will receive structured output in JSON format. The
   log file will not have an ending ']' automatically written to it due to the
   appending nature of logging. It must be appended manually to make the content
   valid JSON.
 
+  A path ending with '.jsonl' will receive structured output in JSON Lines
+  format.
+
 * --masterport:
-  The port on which to contact the puppet master.
-  (This is a Puppet setting, and can go in puppet.conf.)
+  The port on which to contact the Puppet Server.
+  (This is a Puppet setting, and can go in puppet.conf.
+  Deprecated in favor of the 'serverport' setting.)
 
 * --noop:
   Use 'noop' mode where the daemon runs in a no-op or dry-run mode. This
-  is useful for seeing what changes Puppet will make without actually
+  is useful for seeing what changes Puppet would make without actually
   executing the changes.
   (This is a Puppet setting, and can go in puppet.conf. Note the special 'no-'
   prefix for boolean settings on the command line.)
@@ -278,6 +302,10 @@ generated by running puppet agent with '--genconfig'.
   (This is a Puppet setting, and can go in puppet.conf. Note the special 'no-'
   prefix for boolean settings on the command line.)
 
+* --serverport:
+  The port on which to contact the Puppet Server.
+  (This is a Puppet setting, and can go in puppet.conf.)
+
 * --sourceaddress:
   Set the source IP address for transactions. This defaults to automatically selected.
   (This is a Puppet setting, and can go in puppet.conf.)
@@ -286,6 +314,10 @@ generated by running puppet agent with '--genconfig'.
   Enable the most common options used for testing. These are 'onetime',
   'verbose', 'no-daemonize', 'no-usecacheonfailure', 'detailed-exitcodes',
   'no-splay', and 'show_diff'.
+  
+* --trace
+  Prints stack traces on some errors. (This is a Puppet setting, and can go in
+  puppet.conf. Note the special 'no-' prefix for boolean settings on the command line.)
 
 * --verbose:
   Turn on verbose reporting.
@@ -300,9 +332,12 @@ generated by running puppet agent with '--genconfig'.
   it to sign a certificate request. This is useful for the initial setup
   of a puppet client. You can turn off waiting for certificates by
   specifying a time of 0.
-  (This is a Puppet setting, and can go in puppet.conf. Note the special 'no-'
-  prefix for boolean settings on the command line.)
+  (This is a Puppet setting, and can go in puppet.conf.)
 
+* --write_catalog_summary
+  After compiling the catalog saves the resource list and classes list to the node
+  in the state directory named classes.txt and resources.txt
+  (This is a Puppet setting, and can go in puppet.conf.)
 
 EXAMPLE
 -------
@@ -348,12 +383,21 @@ Copyright (c) 2011 Puppet Inc., LLC Licensed under the Apache 2.0 License
 
       log_config if Puppet[:daemonize]
 
-      wait_for_certificates
+      # run ssl state machine, waiting if needed
+      ssl_context = wait_for_certificates
 
-      if Puppet[:onetime]
-        onetime(daemon)
-      else
-        main(daemon)
+      # Each application is responsible for pushing loaders onto the context.
+      # Use the current environment that has already been established, though
+      # it may change later during the configurer run.
+      env = Puppet.lookup(:current_environment)
+      Puppet.override(ssl_context: ssl_context,
+                      current_environment: env,
+                      loaders: Puppet::Pops::Loaders.new(env, true)) do
+        if Puppet[:onetime]
+          onetime(daemon)
+        else
+          main(daemon)
+        end
       end
     end
   end
@@ -369,21 +413,28 @@ Copyright (c) 2011 Puppet Inc., LLC Licensed under the Apache 2.0 License
   end
 
   def fingerprint
-    host = Puppet::SSL::Host.new
-    unless cert = host.certificate || host.certificate_request
-      $stderr.puts _("Fingerprint asked but no certificate nor certificate request have yet been issued")
-      exit(1)
-      return
+    Puppet::Util::Log.newdestination(:console)
+    cert_provider = Puppet::X509::CertProvider.new
+    client_cert = cert_provider.load_client_cert(Puppet[:certname])
+    if client_cert
+      puts Puppet::SSL::Digest.new(options[:digest].to_s, client_cert.to_der).to_s
+    else
+      csr = cert_provider.load_request(Puppet[:certname])
+      if csr
+        puts Puppet::SSL::Digest.new(options[:digest].to_s, csr.to_der).to_s
+      else
+        $stderr.puts _("Fingerprint asked but neither the certificate, nor the certificate request have been issued")
+        exit(1)
+      end
     end
-    unless digest = cert.digest(options[:digest].to_s)
-      raise ArgumentError, _("Could not get fingerprint for digest '%{digest}'") % { digest: options[:digest] }
-    end
-    puts digest.to_s
+  rescue => e
+    Puppet.log_exception(e, _("Failed to generate fingerprint: %{message}") % {message: e.message})
+    exit(1)
   end
 
   def onetime(daemon)
     begin
-      exitstatus = daemon.agent.run(:job_id => options[:job_id])
+      exitstatus = daemon.agent.run({:job_id => options[:job_id], :start_time => options[:start_time]})
     rescue => detail
       Puppet.log_exception(detail)
     end
@@ -440,12 +491,8 @@ Copyright (c) 2011 Puppet Inc., LLC Licensed under the Apache 2.0 License
       Puppet::Resource::Catalog.indirection.cache_class = Puppet[:catalog_cache_terminus]
     end
 
-    if options[:fingerprint]
-      # in fingerprint mode we just need
-      # access to the local files and we don't need a ca
-      Puppet::SSL::Host.ca_location = :none
-    else
-      Puppet::SSL::Host.ca_location = :remote
+    # In fingerprint mode we don't need to set up the whole agent
+    unless options[:fingerprint]
       setup_agent
     end
   end
@@ -470,9 +517,8 @@ Copyright (c) 2011 Puppet Inc., LLC Licensed under the Apache 2.0 License
   end
 
   def daemonize_process_when(should_daemonize)
-    daemon = Puppet::Daemon.new(Puppet::Util::Pidlock.new(Puppet[:pidfile]))
+    daemon = Puppet::Daemon.new(@agent, Puppet::Util::Pidlock.new(Puppet[:pidfile]))
     daemon.argv = @argv
-    daemon.agent = @agent
 
     daemon.daemonize if should_daemonize
 
@@ -480,8 +526,8 @@ Copyright (c) 2011 Puppet Inc., LLC Licensed under the Apache 2.0 License
   end
 
   def wait_for_certificates
-    host = Puppet::SSL::Host.new
     waitforcert = options[:waitforcert] || (Puppet[:onetime] ? 0 : Puppet[:waitforcert])
-    host.wait_for_cert(waitforcert)
+    sm = Puppet::SSL::StateMachine.new(waitforcert: waitforcert)
+    sm.ensure_client_certificate
   end
 end
