@@ -55,7 +55,8 @@ Puppet::Type.type(:package).provide :nim, :parent => :aix, :source => :aix do
   end
 
   def install(useversion = true)
-    unless source = @resource[:source]
+    source = @resource[:source]
+    unless source
       self.fail _("An LPP source location is required in 'source'")
     end
 
@@ -153,20 +154,25 @@ Puppet::Type.type(:package).provide :nim, :parent => :aix, :source => :aix do
   # I spent a lot of time trying to figure out a solution that didn't
   # require parsing the `nimclient -o showres` output and was unable to
   # do so.
-  self::HEADER_LINE_REGEX      = /^([^\s]+)\s+[^@]+@@(I|R):(\1)\s+[^\s]+$/
-  self::PACKAGE_LINE_REGEX     = /^.*@@(I|R):(.*)$/
-  self::RPM_PACKAGE_REGEX      = /^(.*)-(.*-\d+) \2$/
+  self::HEADER_LINE_REGEX      = /^([^\s]+)\s+[^@]+@@(I|R|S):(\1)\s+[^\s]+$/
+  self::PACKAGE_LINE_REGEX     = /^.*@@(I|R|S):(.*)$/
+  self::RPM_PACKAGE_REGEX      = /^(.*)-(.*-\d+\w*) \2$/
   self::INSTALLP_PACKAGE_REGEX = /^(.*) (.*)$/
 
   # Here is some sample output that shows what the above regexes will be up
   # against:
-  # FOR AN INSTALLP PACKAGE:
+  # FOR AN INSTALLP(bff) PACKAGE:
   #
   #    mypackage.foo                                                           ALL  @@I:mypackage.foo _all_filesets
-  #    @ 1.2.3.1  MyPackage Runtime Environment                       @@I:mypackage.foo 1.2.3.1
   #    + 1.2.3.4  MyPackage Runtime Environment                       @@I:mypackage.foo 1.2.3.4
   #    + 1.2.3.8  MyPackage Runtime Environment                       @@I:mypackage.foo 1.2.3.8
   #
+  # FOR AN INSTALLP(bff) PACKAGE with security update:
+  #
+  #    bos.net                                                                 ALL  @@S:bos.net _all_filesets
+  #    + 7.2.0.1  TCP/IP ntp Applications                             @@S:bos.net.tcp.ntp 7.2.0.1
+  #    + 7.2.0.2  TCP/IP ntp Applications                             @@S:bos.net.tcp.ntp 7.2.0.2
+  #
   # FOR AN RPM PACKAGE:
   #
   # mypackage.foo                                                                ALL  @@R:mypackage.foo _all_filesets
@@ -213,7 +219,8 @@ Puppet::Type.type(:package).provide :nim, :parent => :aix, :source => :aix do
   end
 
   def parse_installp_package_string(package_string)
-    unless match = package_string.match(self.class::INSTALLP_PACKAGE_REGEX)
+    match = package_string.match(self.class::INSTALLP_PACKAGE_REGEX)
+    unless match
       self.fail _("Unable to parse output from nimclient showres: package string does not match expected installp package string format:\n'%{package_string}'") % { package_string: package_string }
     end
     package_name = match.captures[0]
@@ -222,7 +229,8 @@ Puppet::Type.type(:package).provide :nim, :parent => :aix, :source => :aix do
   end
 
   def parse_rpm_package_string(package_string)
-    unless match = package_string.match(self.class::RPM_PACKAGE_REGEX)
+    match = package_string.match(self.class::RPM_PACKAGE_REGEX)
+    unless match
       self.fail _("Unable to parse output from nimclient showres: package string does not match expected rpm package string format:\n'%{package_string}'") % { package_string: package_string }
     end
     package_name = match.captures[0]
@@ -231,7 +239,8 @@ Puppet::Type.type(:package).provide :nim, :parent => :aix, :source => :aix do
   end
 
   def parse_showres_package_line(line)
-    unless match = line.match(self.class::PACKAGE_LINE_REGEX)
+    match = line.match(self.class::PACKAGE_LINE_REGEX)
+    unless match
       self.fail _("Unable to parse output from nimclient showres: line does not match expected package line format:\n'%{line}'") % { line: line }
     end
 
@@ -239,7 +248,7 @@ Puppet::Type.type(:package).provide :nim, :parent => :aix, :source => :aix do
     package_string = match.captures[1]
 
     case package_type_flag
-      when "I"
+      when "I","S"
         parse_installp_package_string(package_string)
       when "R"
         parse_rpm_package_string(package_string)
