@@ -9,7 +9,62 @@ module Puppet::Util::POSIX
   # environment for "exec" runs
   USER_ENV_VARS = ['HOME', 'USER', 'LOGNAME']
 
+  class << self
+    # Returns an array of all the groups that the user's a member of.
+    def groups_of(user)
+      begin
+        require 'puppet/ffi/posix'
+        groups = get_groups_list(user)
+      rescue StandardError, LoadError => e
+        Puppet.debug("Falling back to Puppet::Etc.group: #{e.message}")
 
+        groups = []
+        Puppet::Etc.group do |group|
+          groups << group.name if group.mem.include?(user)
+        end
+      end
+
+      uniq_groups = groups.uniq
+      if uniq_groups != groups
+        Puppet.debug(_('Removing any duplicate group entries'))
+      end
+
+      uniq_groups
+    end
+
+    private
+    def get_groups_list(user)
+      raise LoadError, "The 'getgrouplist' method is not available" unless Puppet::FFI::POSIX::Functions.respond_to?(:getgrouplist)
+
+      user_gid = Puppet::Etc.getpwnam(user).gid
+      ngroups = Puppet::FFI::POSIX::Constants::MAXIMUM_NUMBER_OF_GROUPS
+
+      while true do # rubocop:disable Lint/LiteralInCondition
+        FFI::MemoryPointer.new(:int) do |ngroups_ptr|
+          FFI::MemoryPointer.new(:uint, ngroups) do |groups_ptr|
+            old_ngroups = ngroups
+            ngroups_ptr.write_int(ngroups)
+
+            if Puppet::FFI::POSIX::Functions::getgrouplist(user, user_gid, groups_ptr, ngroups_ptr) != -1
+              groups_gids = groups_ptr.get_array_of_uint(0, ngroups_ptr.read_int)
+
+              result = []
+              groups_gids.each do |group_gid|
+                group_info = Puppet::Etc.getgrgid(group_gid)
+                result |= [group_info.name] if group_info.mem.include?(user)
+              end
+              return result
+            end
+
+            ngroups = ngroups_ptr.read_int
+            if ngroups <= old_ngroups
+              ngroups *= 2
+            end
+          end
+        end
+      end
+    end
+  end
 
   # Retrieve a field from a POSIX Etc object.  The id can be either an integer
   # or a name.  This only works for users and groups.  It's also broken on
@@ -17,11 +72,11 @@ module Puppet::Util::POSIX
   # method search_posix_field in the gid and uid methods if a sanity check
   # fails
   def get_posix_field(space, field, id)
-    raise Puppet::DevError, "Did not get id from caller" unless id
+    raise Puppet::DevError, _("Did not get id from caller") unless id
 
     if id.is_a?(Integer)
       if id > Puppet[:maximum_uid].to_i
-        Puppet.err "Tried to get #{field} field for silly id #{id}"
+        Puppet.err _("Tried to get %{field} field for silly id %{id}") % { field: field, id: id }
         return nil
       end
       method = methodbyid(space)
@@ -46,7 +101,7 @@ module Puppet::Util::POSIX
     if id.is_a?(Integer)
       integer = true
       if id > Puppet[:maximum_uid].to_i
-        Puppet.err "Tried to get #{field} field for silly id #{id}"
+        Puppet.err _("Tried to get %{field} field for silly id %{id}") % { field: field, id: id }
         return nil
       end
     end
@@ -74,7 +129,7 @@ module Puppet::Util::POSIX
     when :gr, :group; return :gid
     when :pw, :user, :passwd; return :uid
     else
-      raise ArgumentError.new("Can only handle users and groups")
+      raise ArgumentError.new(_("Can only handle users and groups"))
     end
   end
 
@@ -84,7 +139,7 @@ module Puppet::Util::POSIX
     when :gr, :group; return :getgrgid
     when :pw, :user, :passwd; return :getpwuid
     else
-      raise ArgumentError.new("Can only handle users and groups")
+      raise ArgumentError.new(_("Can only handle users and groups"))
     end
   end
 
@@ -94,7 +149,7 @@ module Puppet::Util::POSIX
     when :gr, :group; return :getgrnam
     when :pw, :user, :passwd; return :getpwnam
     else
-      raise ArgumentError.new("Can only handle users and groups")
+      raise ArgumentError.new(_("Can only handle users and groups"))
     end
   end
 
@@ -119,16 +174,27 @@ module Puppet::Util::POSIX
       # pass
     end
     if field.is_a?(Integer)
-      return nil unless name = get_posix_field(location, :name, field)
+      name = get_posix_field(location, :name, field)
+      return nil unless name
       id = get_posix_field(location, id_field, name)
       check_value = id
     else
-      return nil unless id = get_posix_field(location, id_field, field)
+      id = get_posix_field(location, id_field, field)
+      return nil unless id
       name = get_posix_field(location, :name, id)
       check_value = name
     end
+
     if check_value != field
-      return search_posix_field(location, id_field, field)
+      check_value_id = get_posix_field(location, id_field, check_value) if check_value
+
+      if id == check_value_id
+        Puppet.debug("Multiple entries found for resource: '#{location}' with #{id_field}: #{id}")
+        return id
+      else
+        Puppet.debug("The value retrieved: '#{check_value}' is different than the required state: '#{field}', searching in all entries")
+        return search_posix_field(location, id_field, field)
+      end
     else
       return id
     end
